@@ -1,6 +1,7 @@
 import json
 import warnings
 
+from django.utils.version import PY36, get_docs_version
 from typing import Callable, Optional, TypeVar, Any
 from django.core.cache import cache
 
@@ -9,11 +10,25 @@ _NOT_FOUND = object()
 
 
 class splint_cached_property:
+    name = None
+
+    @staticmethod
+    def func(instance):
+        raise TypeError(
+            'Cannot use cached_property instance without calling '
+            '__set_name__() on it.'
+        )
+
+    @staticmethod
+    def _is_mangled(name):
+        return name.startswith('__') and not name.endswith('__')
+
     def __init__(
         self,
         func: Callable[..., _T],
+        name: str = None,
         cache_key: Callable[..., _T] = None,
-        cache_expires: Optional[int] = None,
+        cache_expires: Optional[int] = 60 * 60 * 3,
     ):
         """Class to saves properties in cache services.
 
@@ -23,14 +38,15 @@ class splint_cached_property:
         Args:
             func (Callable[..., _T]): Callable should be return any 
                 picklable Python object.
+            name: (Optional[str], , optional): The name of attribute optional,
+                by default it will be '{func.__name__}'.
             cache_key (Callable[..., _T], optional): Callable should be return a sha str. 
                 Defaults call to '{func.__name__}__cache_key'.
             cache_expires (Optional[int], optional): The timeout argument is 
-                optional and defaults to the timeout argument of the appropriate 
-                backend in the CACHES setting. 
+                optional.
                 Its the number of seconds the value should be stored in the cache. 
                 A timeout of 0 wont cache the value. 
-                Defaults to None for timeout will cache the value forever.
+                Defaults to three hours for expires cache value.
 
         Raises:
             TypeError: Cannot assign the same splint_cached_property to two different names.
@@ -41,27 +57,49 @@ class splint_cached_property:
         Returns:
             Picklable: picklable Python object.
         """
-        self.func = func
+
+        if PY36:
+            self.real_func = func
+        else:
+            name = name or func.__name__
+            if not (isinstance(name, str) and name.isidentifier()):
+                raise ValueError(
+                    "%r can't be used as the name of a cached property." % name,
+                )
+
+            if self._is_mangled(name):
+                raise ValueError(
+                    'cached property does not work with mangled methods on '
+                    'Python < 3.6 without the appropriate `name` argument. See '
+                    'https://docs.djangoproject.com/en/%s/ref/utils/'
+                    '#cached-property-mangled-name' % get_docs_version(),
+                )
+
+            self.name = name
+            self.func = func
+
         self.cache_key = cache_key
-        self.attrname = None
         self.cache_expires = cache_expires
         self.__doc__ = getattr(func, '__doc__')
 
     def __set_name__(self, owner, name):
-        if self.attrname is None:
-            self.attrname = self.func.__name__
-        elif name != self.attrname:
+        if self.name is None:
+            self.name = name
+            self.func = self.real_func
+        elif name != self.name:
             raise TypeError(
                 "Cannot assign the same splint_cached_property to two different names "
-                f"({self.attrname!r} and {name!r})."
+                f"({self.name!r} and {name!r})."
             )
 
     def __get__(self, instance, cls=None) -> Any:
         if instance is None:
             return self
-        if self.attrname is None:
+
+        if self.name is None:
             raise TypeError(
-                "Cannot use splint_cached_property instance without calling __set_name__ on it.")
+                "Cannot use splint_cached_property " +
+                "instance without calling __set_name__ on it.")
 
         try:
             instance_cache = instance.__dict__
@@ -69,21 +107,21 @@ class splint_cached_property:
         except AttributeError:
             msg = (
                 f"No '__dict__' attribute on {type(instance).__name__!r} "
-                f"instance to cache {self.attrname!r} property."
+                f"instance to cache {self.name!r} property."
             )
             raise TypeError(msg) from None
 
-        cache_value = instance_cache.get(self.attrname, _NOT_FOUND)
+        cache_value = instance_cache.get(self.name, _NOT_FOUND)
 
         if cache_value is _NOT_FOUND:
             if self.cache_key is None:
                 try:
                     cache_key = getattr(
-                        instance, f'{self.attrname}__cache_key')()
+                        instance, f'{self.name}__cache_key')()
                 except AttributeError:
                     msg = (
-                        f"No '{self.attrname}__cache_key' attribute on instance to cache "
-                        f"{self.attrname!r} property."
+                        f"No '{self.name}__cache_key' attribute on instance to cache "
+                        f"{self.name!r} property."
                     )
                     raise TypeError(msg) from None
             else:
@@ -99,15 +137,18 @@ class splint_cached_property:
                 timeout=self.cache_expires)
 
             try:
-                instance_cache[self.attrname] = cache_value
+                instance_cache[self.name] = cache_value
             except TypeError:
                 msg = (
                     f"The '__dict__' attribute on {type(instance).__name__!r} instance "
-                    f"does not support item assignment for caching {self.attrname!r} property."
+                    f"does not support item assignment for caching {self.name!r} property."
                 )
                 warnings.warn(msg, Warning)
 
         if not isinstance(cache_value, dict):
-            cache_value = json.loads(cache_value)
+            try:
+                cache_value = json.loads(cache_value)
+            except:
+                pass
 
         return cache_value
